/* Sistemas Integrados TECNISA
   Planejamento Alimentao Industrial

Nome: VERIFICA_CUSTO.SQL

Descrio:
Stored Procedure auxiliar utilizada na funo Custo_do_Produto.
Faz os clculos e retorna os custos.

P_CDFILIAL     - Cdigo da Filial.
P_CDPRODUTO    - Cdigo do Produto.
P_NRTIPOCUST   - Nmero do Tipo de Custo (1 a 10)
P_DTREFERENCIA - Data do Custo.
P_VRULTICUST   - Recebe o valor calculado para ltimo custo de compras.
P_VRULTICOTA   - Recebe o valor calculado para ltimo custo de cotao.
P_VRCUSTMEDI   - Recebe o valor calculado para custo mdio.

TIPOS DE CUSTO:

01 - ltimo custo de compra
02 - Ultimo custo atualizado
03 - ltimo custo (em moeda forte)
04 - Ultimo custo de cotao (AF)
05 - Custo mdio (lquido) atual
06 - Custo mdio (lquido) da poca
07 - Custo mdio (lquido em moeda forte)
08 - Custo mdio (bruto) atual
09 - Custo mdio (bruto) da poca
10 - Custo mdio (bruto em moeda forte)
11 - Custo de Cotao (Lista de Preos)

@C:\Win_prj\Sql\Oracle\0311_PLASTP_VERIFICA_CUSTO.SQL

Criao:
12/08/98 - Ederson

Manutenes:
18/08/99 - Roberto - Parmetros da Procedure POSICAO_ESTOQUE.

*/

CREATE OR REPLACE
PROCEDURE VERIFICA_CUSTO
   (P_CDFILIAL     IN FILIAL.CDFILIAL%TYPE,
    P_CDPRODUTO    IN PRODUTO.CDPRODUTO%TYPE,
    P_NRTIPOCUST   IN NUMBER,
    P_DTREFERENCIA IN DATE,
    P_VRULTICUST   IN OUT NUMBER,
    P_VRULTICOTA   IN OUT NUMBER,
    P_VRCUSTMEDI   IN OUT NUMBER,
    P_ALMOXARIFE   IN ALMOXARIFADO.CDALMOXARIFE%TYPE DEFAULT NULL,
    P_LOCALESTOQ   IN LOCALESTOQUE.CDLOCALESTOQ%TYPE DEFAULT NULL,
    P_NRLOTEESTQ   IN POSESTDIA.NRLOTEESTQ%TYPE DEFAULT NULL,
    P_NRSUBLOTE    IN POSESTDIA.NRSUBLOTE%TYPE  DEFAULT NULL) IS

   V_DOLAREPOCA NUMBER;
   V_DOLARATUAL NUMBER;

   V_DTPOSICAO  DATE;
   V_DTPOSIAUXI DATE;

   V_QTDEESTO   NUMBER;
   V_VALRESTO   NUMBER;
   V_VALRBRUT   NUMBER;

   V_DTPROCESSA DATE;

   V_VRULTCOMPR   NUMBER;
   V_DTULTCOMPR   DATE;
   V_VRPRECUAFOFI NUMBER;

   V_CDINDICEMF PARAMGERAL.CDINDICEMF%TYPE;

   V_DTREFE DATE;

   V_VRINDICE NUMBER;

   V_CDFILCENTCUST VARCHAR2(4);
   V_CDREGIAO VARCHAR2(3);
   V_VRLISTAPRECO NUMBER;

   /* - Declarao de cursores ---------------------------------------------- */

   CURSOR C_DATAPROC IS
      SELECT
         NVL(DTPROCESSA, SYSDATE) AS DTPROCESSA
      FROM
         PARAMFILIAL A, PARAMGERAL B
      WHERE
         (A.CDFILIAL = P_CDFILIAL) AND
         (B.IDUTILDTPROC <> 'N');

   CURSOR C_FIND_PRODFILI IS
      SELECT
         VRULTCOMPR,
         DTULTCOMPR,
         VRPRECUAFOFI
      FROM
         PRODFILI
      WHERE
         (CDFILIAL  = P_CDFILIAL) AND
         (CDPRODUTO = P_CDPRODUTO);

   CURSOR C_CDINDICEMF IS
      SELECT
         CDINDICEMF
      FROM
         PARAMGERAL;

   CURSOR C_FIND_MOVINDDI IS
      SELECT
         VRINDICE
      FROM
         MOVINDICE
      WHERE
         (CDTPINDICE = '1')    AND
         (DTREFIND = V_DTREFE) AND
         (CDINDICE = V_CDINDICEMF);

   CURSOR C_FIND_POSESTDIA IS
      SELECT
         MAX(DTPOSIESTQ)
      FROM
         POSESTDIA
      WHERE
         (CDFILIAL    = P_CDFILIAL)  AND
         (CDPRODUTO   = P_CDPRODUTO) AND
         (QTESTOQDIA  > 0)           AND
         (DTPOSIESTQ <= V_DTPOSICAO);

BEGIN

   /* - Verifica a data de Processamento da Filial -------------------------- */

   OPEN  C_DATAPROC;
   FETCH C_DATAPROC
   INTO V_DTPROCESSA;
   IF C_DATAPROC%NOTFOUND THEN
      SELECT SYSDATE
      INTO V_DTPROCESSA
      FROM DUAL;
   END IF;
   CLOSE C_DATAPROC;

   
   /* ----------------------------------------------------------------------- */

   P_VRULTICUST := 0;
   P_VRULTICOTA := 0;
   P_VRCUSTMEDI := 0;

   IF P_NRTIPOCUST IN (1, 2, 3) THEN

      P_VRULTICUST := 0;

      OPEN  C_CDINDICEMF;
      FETCH C_CDINDICEMF
      INTO V_CDINDICEMF;
      IF C_CDINDICEMF%NOTFOUND THEN
         V_CDINDICEMF := ' ';
      END IF;
      CLOSE C_CDINDICEMF;

      OPEN  C_FIND_PRODFILI;
      FETCH C_FIND_PRODFILI
      INTO V_VRULTCOMPR,
           V_DTULTCOMPR,
           V_VRPRECUAFOFI;
      IF C_FIND_PRODFILI%NOTFOUND THEN
         V_VRULTCOMPR   := 0;
         V_DTULTCOMPR   := TO_DATE('01 01 0001', 'DD MM YYYY');
         V_VRPRECUAFOFI := 0;
      END IF;
      CLOSE C_FIND_PRODFILI;


      IF P_NRTIPOCUST IN (2, 3) THEN
         V_DTREFE   := V_DTULTCOMPR;
         OPEN  C_FIND_MOVINDDI;
         FETCH C_FIND_MOVINDDI
         INTO V_VRINDICE;
         IF C_FIND_MOVINDDI%NOTFOUND THEN
            V_VRINDICE := 0;
         END IF;
         CLOSE C_FIND_MOVINDDI;
         V_DOLAREPOCA := V_VRINDICE;



         V_DTREFE   := V_DTPROCESSA;
         OPEN  C_FIND_MOVINDDI;
         FETCH C_FIND_MOVINDDI
         INTO V_VRINDICE;
         IF C_FIND_MOVINDDI%NOTFOUND THEN
            V_VRINDICE := 0;
         END IF;
         CLOSE C_FIND_MOVINDDI;
         V_DOLARATUAL := V_VRINDICE;

         IF V_DOLAREPOCA = 0 THEN
            V_DOLAREPOCA := V_DOLARATUAL;
         END IF;
      END IF;

      P_VRULTICUST := 0;

      IF P_NRTIPOCUST = 1 THEN
         P_VRULTICUST := V_VRULTCOMPR;
      END IF;
      IF P_NRTIPOCUST = 2 THEN
         IF V_DOLAREPOCA <> 0 THEN
            P_VRULTICUST := (V_VRULTCOMPR/V_DOLAREPOCA) * V_DOLARATUAL;
         END IF;
      END IF;
      IF P_NRTIPOCUST = 3 THEN
         IF V_DOLAREPOCA <> 0 THEN
            P_VRULTICUST := V_VRULTCOMPR/V_DOLAREPOCA;
         END IF;
      END IF;
   END IF;

   IF P_NRTIPOCUST = 4 THEN
      P_VRULTICOTA   := 0;

      OPEN  C_FIND_PRODFILI;
      FETCH C_FIND_PRODFILI
      INTO V_VRULTCOMPR,
           V_DTULTCOMPR,
           V_VRPRECUAFOFI;
      IF C_FIND_PRODFILI%NOTFOUND THEN
         V_VRULTCOMPR   := 0;
         V_DTULTCOMPR   := TO_DATE('01 01 0001', 'DD MM YYYY');
         V_VRPRECUAFOFI := 0;
      END IF;
      CLOSE C_FIND_PRODFILI;

      P_VRULTICOTA := V_VRPRECUAFOFI;
   END IF;


   IF P_NRTIPOCUST IN (5,6,7,8,9,10) THEN

      P_VRCUSTMEDI := 0;

      IF P_NRTIPOCUST IN (5,7,8,10) THEN
         V_DTPOSICAO := V_DTPROCESSA + 1;
      END IF;
      IF P_NRTIPOCUST IN (6,9) THEN
         V_DTPOSICAO := P_DTREFERENCIA + 1;
      END IF;

      POSICAO_ESTOQUE(P_CDFILIAL,
                      P_CDPRODUTO,
                      V_DTPOSICAO,
                      V_QTDEESTO,
                      V_VALRESTO,
                      V_VALRBRUT,
                      'S',
                      P_ALMOXARIFE,
                      P_LOCALESTOQ,
                      NULL,
                      NULL,
                      NULL,
                      P_NRLOTEESTQ,
                      P_NRSUBLOTE);

      /* Lquido */
      IF P_NRTIPOCUST IN (5,6,7) THEN
         IF V_QTDEESTO <> 0 THEN
            P_VRCUSTMEDI := V_VALRESTO / V_QTDEESTO;
         ELSE
            P_VRCUSTMEDI := 0;
         END IF;
      END IF;
      /* Bruto */
      IF P_NRTIPOCUST IN (8,9,10) THEN
         IF V_QTDEESTO <> 0 THEN
            P_VRCUSTMEDI := V_VALRBRUT / V_QTDEESTO;
         ELSE
            P_VRCUSTMEDI := 0;
         END IF;
      END IF;

      IF P_NRTIPOCUST IN (7,10) THEN

         OPEN  C_CDINDICEMF;
         FETCH C_CDINDICEMF
         INTO V_CDINDICEMF;
         IF C_CDINDICEMF%NOTFOUND THEN
            V_CDINDICEMF := ' ';
         END IF;
         CLOSE C_CDINDICEMF;

         V_DTREFE   := V_DTPROCESSA;
         OPEN  C_FIND_MOVINDDI;
         FETCH C_FIND_MOVINDDI
         INTO V_VRINDICE;
         IF C_FIND_MOVINDDI%NOTFOUND THEN
            V_VRINDICE := 0;
         END IF;
         CLOSE C_FIND_MOVINDDI;

         IF V_VRINDICE <> 0 THEN
            P_VRCUSTMEDI := (P_VRCUSTMEDI/V_VRINDICE);
         ELSE
            P_VRCUSTMEDI := 0;
         END IF;

      END IF;
   END IF;

   /* CUSTO DE COTACAO - LISTA DE PRECOS - Calculos identicos a LIB00500 */
   IF P_NRTIPOCUST IN (11) THEN
      SELECT
         CDFILCENTCMP,
         CDREGIAO
      INTO
         V_CDFILCENTCUST,
         V_CDREGIAO
      FROM
         FILIAL
      WHERE
         CDFILIAL = P_CDFILIAL;

      IF (V_CDFILCENTCUST IS NULL) OR (V_CDFILCENTCUST = '') THEN
         V_CDFILCENTCUST := P_CDFILIAL;
      END IF;

      SELECT AVG(ITEMCOLE.VRPRECUNIITM/PRODUTO.VRFATOCONV)
      INTO V_VRLISTAPRECO
      FROM ITEMCOLE, PRODUTO
      WHERE
        ITEMCOLE.IDITMLISPREC = 'S'               AND
        ITEMCOLE.CDFILIAL     = V_CDFILCENTCUST   AND
        ITEMCOLE.CDREGIAO     = V_CDREGIAO        AND
        PRODUTO.CDPRODESTO    = P_CDPRODUTO       AND
        ITEMCOLE.CDPRODUTO    = PRODUTO.CDPRODUTO AND
        ITEMCOLE.DTINIVALITM  = (SELECT MAX(I.DTINIVALITM)
                                 FROM   ITEMCOLE I
                                 WHERE
                                   I.IDITMLISPREC = 'S'                   AND
                                   ITEMCOLE.CDFILIAL    = V_CDFILCENTCUST AND
                                   ITEMCOLE.CDREGIAO    = V_CDREGIAO      AND
                                   ITEMCOLE.CDFORNECED  = I.CDFORNECED    AND
                                   ITEMCOLE.CDPRODUTO   = I.CDPRODUTO     AND
                                   ITEMCOLE.CDMARCA     = I.CDMARCA);

      IF V_VRLISTAPRECO IS NULL THEN
         V_VRLISTAPRECO := 0;
      END IF;

      P_VRULTICOTA := V_VRLISTAPRECO;

   END IF;

   IF P_VRULTICUST IS NULL THEN
      P_VRULTICUST := 0;
   END IF;
   IF P_VRULTICOTA IS NULL THEN
      P_VRULTICOTA := 0;
   END IF;
   IF P_VRCUSTMEDI IS NULL THEN
      P_VRCUSTMEDI := 0;
   END IF;
END;
/
