CREATE OR REPLACE PACKAGE PEDIDO_COLETA IS

   /**
    * Package responsvel por tratar todos os processos que envolvem a rotina de Pedido de Coleta.
    *
    * @autor Joo Paulo Rodrigues Crte
    * @data 06/03/2023
    *
    */

    /* Constantes */
    K_A   CONSTANT VARCHAR2(01) := 'A';
    K_C   CONSTANT VARCHAR2(01) := 'C';
    K_D   CONSTANT VARCHAR2(01) := 'D';
    K_M   CONSTANT VARCHAR2(01) := 'M';
    K_N   CONSTANT VARCHAR2(01) := 'N';
    K_O   CONSTANT VARCHAR2(01) := 'O';
    K_R   CONSTANT VARCHAR2(01) := 'R';
    K_S   CONSTANT VARCHAR2(01) := 'S';
    K_Z   CONSTANT VARCHAR2(01) := 'Z';
    K_CP  CONSTANT VARCHAR2(01) := '%';

    /* Types e Subtypes */
    SUBTYPE TYPE_PEDIDOCOLETATMP IS PEDIDOCOLETA_TMP%ROWTYPE;
    TYPE TABLE_PEDIDOCOLETATMP   IS TABLE OF TYPE_PEDIDOCOLETATMP;

    SUBTYPE S_RECORD_PEDIDOCOLETA_TMP IS PEDIDOCOLETA_TMP%ROWTYPE;
    SUBTYPE S_RECORD_PARASUPRI        IS PARASUPRI%ROWTYPE;
    SUBTYPE S_RECORD_FILIAL           IS FILIAL%ROWTYPE;
    
    /*
     * Procedure responsvel por buscar as informaes da tabela PARASUPRI
     *
     * @entrada P_CDFILIAL:  Cdigo da unidade
     * @saida   P_PARASUPRI: Registro da tabela PARASUPRI
     */
    PROCEDURE GET_PARASUPRI(P_CDFILIAL   IN FILIAL.CDFILIAL%TYPE,
                            P_PARASUPRI OUT S_RECORD_PARASUPRI);

    /*
     * Procedure responsvel por buscar as informaes da tabela FILIAL
     *
     * @entrada P_CDFILIAL: Cdigo da unidade
     * @saida   P_FILIAL:   Registro da tabela FILIAL
     */
    PROCEDURE GET_DADOSFILIAL(P_CDFILIAL IN FILIAL.CDFILIAL%TYPE,
                              P_FILIAL  OUT S_RECORD_FILIAL);

    /*
     * Procedure responsvel por inserir registros a tabela PEDIDOCOLETA_TMP
     *
     * @entrada P_ITEMS: Registro com dados do pedido de coleta
     */
    PROCEDURE INSERT_PEDIDOCOLETATMP(P_ITEMS IN S_RECORD_PEDIDOCOLETA_TMP);

    /* 
     * Funo responsvel por montar, em tempo de excuo, a query dos pedidos de coleta A Comprar/Todos.
     *
     * @entrada P_PARASUPRI:   Registro da tabela PARASUPRI
     * @entrada P_FILIAL:      Registro da tabela FILIAL
     * @entrada P_CDFILIPROC:  Cdigo da unidade de processamento
     * @entrada P_CDFILIAL:    Cdigo da unidade
     * @entrada P_CDMARCA:     Cdigo da(s) marca(s)
     * @entrada P_CDCOMPRADOR: Cdigo do(s) comprador(es)
     * @entrada P_CDFORNECED:  Cdigo do(s) fornecedor(es)
     * @entrada P_CDREGIAO:    Cdigo da(s) regio(es)
     * @entrada P_CDPRODINI:   Cdigo do produto inicial (CDARVPROD)
     * @entrada P_CDPRODFIN:   Cdigo do produto final (CDARVPROD)
     * @entrada P_DTINI:       Data inicial
     * @entrada P_DTFIN:       Data final
     * @entrada P_TIPOPEDIDO:  Tipo do pedido (COMPRAR ou TODOS)
     */
    FUNCTION MONTA_TODOS_ACOMPRAR(P_PARASUPRI   IN S_RECORD_PARASUPRI,
                                  P_FILIAL      IN S_RECORD_FILIAL, 
                                  P_CDFILIPROC  IN FILIAL.CDFILIAL%TYPE,
                                  P_CDFILIAL    IN FILIAL.CDFILIAL%TYPE,
                                  P_CDMARCA     IN VARCHAR2,
                                  P_CDCOMPRADOR IN VARCHAR2,
                                  P_CDFORNECED  IN VARCHAR2,
                                  P_CDREGIAO    IN VARCHAR2,
                                  P_CDPRODINI   IN PRODUTO.CDARVPROD%TYPE,
                                  P_CDPRODFIN   IN PRODUTO.CDARVPROD%TYPE,
                                  P_DTINI       IN DATE,
                                  P_DTFIN       IN DATE,
                                  P_TIPOPEDIDO  IN VARCHAR2) RETURN VARCHAR2;

    /* 
     * Funo responsvel por montar, em tempo de excuo, a query dos pedidos de coleta por cotao.
     *
     * @entrada P_PARASUPRI:   Registro da tabela PARASUPRI
     * @entrada P_FILIAL:      Registro da tabela FILIAL
     * @entrada P_CDFILIPROC:  Cdigo da unidade de processamento
     * @entrada P_CDFILIAL:    Cdigo da unidade
     * @entrada P_CDMARCA:     Cdigo da(s) marca(s)
     * @entrada P_CDCOMPRADOR: Cdigo do(s) comprador(es)
     * @entrada P_CDFORNECED:  Cdigo do(s) fornecedor(es)
     * @entrada P_CDREGIAO:    Cdigo da(s) regio(es)
     * @entrada P_CDPRODINI:   Cdigo do produto inicial (CDARVPROD)
     * @entrada P_CDPRODFIN:   Cdigo do produto final (CDARVPROD)
     * @entrada P_DTINI:       Data inicial
     * @entrada P_DTFIN:       Data final
     * @entrada P_NRCOLEPRECO: Cdigo da coleta de preos (Planilha de cotao)
     * @entrada P_DESCFORNCOT: Identifica se o fornecedor da cotao ser considerado (opes S ou N)
     */
    FUNCTION MONTA_COTACAO(P_PARASUPRI   IN S_RECORD_PARASUPRI,
                           P_FILIAL      IN S_RECORD_FILIAL, 
                           P_CDFILIPROC  IN FILIAL.CDFILIAL%TYPE,
                           P_CDFILIAL    IN FILIAL.CDFILIAL%TYPE,
                           P_CDMARCA     IN VARCHAR2,
                           P_CDCOMPRADOR IN VARCHAR2,
                           P_CDFORNECED  IN VARCHAR2,
                           P_CDREGIAO    IN VARCHAR2,
                           P_CDPRODINI   IN PRODUTO.CDARVPROD%TYPE,
                           P_CDPRODFIN   IN PRODUTO.CDARVPROD%TYPE,
                           P_NRCOLEPRECO IN ITEMCOLE.NRCOLEPRECO%TYPE,
                           P_DESCFORNCOT IN VARCHAR2 DEFAULT 'N') RETURN VARCHAR2;

    /* 
     * Funo responsvel por montar, em tempo de excuo, a query dos pedidos de coleta por lista de preos,
     * para clientes que utilizam o campo PARAMGERAL.IDSISDOS com valor igual a 3
     *
     * @entrada P_PARASUPRI:    Registro da tabela PARASUPRI
     * @entrada P_FILIAL:       Registro da tabela FILIAL
     * @entrada P_CDFILIPROC:   Cdigo da unidade de processamento
     * @entrada P_CDFILIAL:     Cdigo da unidade
     * @entrada P_CDMARCA:      Cdigo da(s) marca(s)
     * @entrada P_CDCOMPRADOR:  Cdigo do(s) comprador(es)
     * @entrada P_CDFORNECED:   Cdigo do(s) fornecedor(es)
     * @entrada P_CDREGIAO:     Cdigo da(s) regio(es)
     * @entrada P_CDPRODINI:    Cdigo do produto inicial (CDARVPROD)
     * @entrada P_CDPRODFIN:    Cdigo do produto final (CDARVPROD)
     * @entrada P_NRSEQPRELIST: Cdigo da lista de preos
     * @entrada P_DTINI:        Data inicial
     * @entrada P_DTFIN:        Data final
     */
    FUNCTION MONTA_LISTPREC_IDSISDOS_3(P_PARASUPRI    IN S_RECORD_PARASUPRI,
                                       P_FILIAL       IN S_RECORD_FILIAL,
                                       P_CDFILIPROC   IN FILIAL.CDFILIAL%TYPE,
                                       P_CDFILIAL     IN FILIAL.CDFILIAL%TYPE,
                                       P_CDMARCA      IN VARCHAR2,
                                       P_CDCOMPRADOR  IN VARCHAR2,
                                       P_CDFORNECED   IN VARCHAR2,
                                       P_CDREGIAO     IN VARCHAR2,
                                       P_CDPRODINI    IN PRODUTO.CDARVPROD%TYPE,
                                       P_CDPRODFIN    IN PRODUTO.CDARVPROD%TYPE,
                                       P_NRSEQPRELIST IN ITEMCOLE.NRSEQPRELIST%TYPE,
                                       P_DTINI        IN DATE,
                                       P_DTFIN        IN DATE) RETURN VARCHAR2;

    /* 
     * Funo responsvel por montar, em tempo de excuo, a query dos pedidos de coleta por lista de preos,
     * para clientes que utilizam o campo PARAMGERAL.IDSISDOS com valor igual a 1
     *
     * @entrada P_PARASUPRI:    Registro da tabela PARASUPRI
     * @entrada P_FILIAL:       Registro da tabela FILIAL
     * @entrada P_CDFILIPROC:   Cdigo da unidade de processamento
     * @entrada P_CDFILIAL:     Cdigo da unidade
     * @entrada P_CDMARCA:      Cdigo da(s) marca(s)
     * @entrada P_CDCOMPRADOR:  Cdigo do(s) comprador(es)
     * @entrada P_CDFORNECED:   Cdigo do(s) fornecedor(es)
     * @entrada P_CDREGIAO:     Cdigo da(s) regio(es)
     * @entrada P_CDPRODINI:    Cdigo do produto inicial (CDARVPROD)
     * @entrada P_CDPRODFIN:    Cdigo do produto final (CDARVPROD)
     * @entrada P_NRSEQPRELIST: Cdigo da lista de preos
     */
    FUNCTION MONTA_LISTPREC_IDSISDOS_1(P_PARASUPRI    IN S_RECORD_PARASUPRI,
                                       P_FILIAL       IN S_RECORD_FILIAL,
                                       P_CDFILIPROC   IN FILIAL.CDFILIAL%TYPE,
                                       P_CDFILIAL     IN FILIAL.CDFILIAL%TYPE,
                                       P_CDMARCA      IN VARCHAR2,
                                       P_CDCOMPRADOR  IN VARCHAR2,
                                       P_CDFORNECED   IN VARCHAR2,
                                       P_CDREGIAO     IN VARCHAR2,
                                       P_CDPRODINI    IN PRODUTO.CDARVPROD%TYPE,
                                       P_CDPRODFIN    IN PRODUTO.CDARVPROD%TYPE,
                                       P_NRSEQPRELIST IN ITEMCOLE.NRSEQPRELIST%TYPE) RETURN VARCHAR2;

    /* 
     * Funo responsvel por montar, em tempo de excuo, a query dos pedidos de coleta por nro de compra.
     *
     * @entrada P_PARASUPRI:   Registro da tabela PARASUPRI
     * @entrada P_FILIAL:      Registro da tabela FILIAL
     * @entrada P_CDFILIPROC:  Cdigo da unidade de processamento
     * @entrada P_CDFILIAL:    Cdigo da unidade
     * @entrada P_CDMARCA:     Cdigo da(s) marca(s)
     * @entrada P_CDCOMPRADOR: Cdigo do(s) comprador(es)
     * @entrada P_CDFORNECED:  Cdigo do(s) fornecedor(es)
     * @entrada P_CDREGIAO:    Cdigo da(s) regio(es)
     * @entrada P_CDPRODINI:   Cdigo do produto inicial (CDARVPROD)
     * @entrada P_CDPRODFIN:   Cdigo do produto final (CDARVPROD)
     * @entrada P_NRCOMPRA:    Cdigo da compra
     * @entrada P_DTINI:       Data inicial
     * @entrada P_DTFIN:       Data final
     */
    FUNCTION MONTA_NRCOMPRA(P_PARASUPRI   IN S_RECORD_PARASUPRI,
                            P_FILIAL      IN S_RECORD_FILIAL,
                            P_CDFILIPROC  IN FILIAL.CDFILIAL%TYPE,
                            P_CDFILIAL    IN FILIAL.CDFILIAL%TYPE,
                            P_CDMARCA     IN VARCHAR2,
                            P_CDCOMPRADOR IN VARCHAR2,
                            P_CDFORNECED  IN VARCHAR2,
                            P_CDREGIAO    IN VARCHAR2,
                            P_CDPRODINI   IN PRODUTO.CDARVPROD%TYPE,
                            P_CDPRODFIN   IN PRODUTO.CDARVPROD%TYPE,
                            P_NRCOMPRA    IN COMPRA.NRCOMPRA%TYPE,
                            P_DTINI       IN DATE,
                            P_DTFIN       IN DATE) RETURN VARCHAR2;

    /*
     * Procedure responsvel por buscar as informaes do pedido de coleta
     *
     * @entrada P_CDFILIAL:     Cdigo da unidade
     * @entrada P_CDFILIPROC:   Cdigo da unidade de processamento
     * @entrada P_CDCOMPRADOR:  Cdigo do(s) comprador(es)
     * @entrada P_CDPRODINI:    Cdigo do produto inicial (CDARVPROD)
     * @entrada P_CDPRODFIN:    Cdigo do produto final (CDARVPROD)
     * @entrada P_CDFORNECED:   Cdigo do(s) fornecedor(es)
     * @entrada P_CDREGIAO:     Cdigo da(s) regio(es)
     * @entrada P_CDMARCA:      Cdigo da(s) marca(s)
     * @entrada P_DTINI:        Data inicial
     * @entrada P_DTFIN:        Data final
     * @entrada P_TIPOPEDIDO:   Tipo do pedido (a comprar ou todos)
     * @entrada P_NRCOLEPRECO:  Cdigo da coleta de preos (Planilha de cotao)
     * @entrada P_DESCFORNCOT:  Identifica se o fornecedor da cotao ser considerado (opes S ou N)
     * @entrada P_NRCOMPRA:     Cdigo da compra
     * @entrada P_CDLEILAO:     Cdigo do leilo
     * @entrada P_NRSEQPRELIST: Cdigo da lista de preos
     * @entrada P_CDOPERLANC:   Cdigo do operador de lanamento
     * @entrada P_NRORG:        Cdigo da oranizao
     * @saida   P_NRPEDCOL:     Cdigo do pedido de coleta
     * @saida   P_ERROR:        Cdigo/descrio possvel inconsistncia
     */
    PROCEDURE GETDATAPEDIDOCOLETA(P_CDFILIAL     IN FILIAL.CDFILIAL%TYPE,
                                  P_CDFILIPROC   IN FILIAL.CDFILIAL%TYPE,
                                  P_CDCOMPRADOR  IN VARCHAR2,
                                  P_CDPRODINI    IN VARCHAR2,
                                  P_CDPRODFIN    IN VARCHAR2,
                                  P_CDFORNECED   IN VARCHAR2,
                                  P_CDREGIAO     IN VARCHAR2,
                                  P_CDMARCA      IN VARCHAR2,
                                  P_DTINI        IN DATE,
                                  P_DTFIN        IN DATE,
                                  P_TIPOPEDIDO   IN VARCHAR2,
                                  P_NRCOLEPRECO  IN ITEMCOLE.NRCOLEPRECO%TYPE,
                                  P_DESCFORNCOT  IN VARCHAR2,
                                  P_NRCOMPRA     IN ITEMCOLE.NRCOLEPRECO%TYPE,
                                  P_CDLEILAO     IN PEDCOLETA.CDLEILAO%TYPE,
                                  P_NRSEQPRELIST IN ITEMCOLE.NRSEQPRELIST%TYPE,
                                  P_CDOPERLANC   IN OPERADOR.CDOPERADOR%TYPE,
                                  P_NRORG        IN FILIAL.NRORG%TYPE,
                                  P_NRPEDCOL     IN OUT PEDIDOCOLETA_TMP.NRPEDCOL%TYPE,
                                  P_ERROR       OUT VARCHAR2);

    /*
     * Procedure responsvel por acrescentar itens contidos na previso de consumo do perodo
     *
     * @entrada P_CDFILIAL:    Cdigo da unidade
     * @entrada P_CDPRODINI:   Cdigo do produto inicial (CDARVPROD)
     * @entrada P_CDPRODFIN:   Cdigo do produto final (CDARVPROD)
     * @entrada P_CDMARCA:     Cdigo da(s) marca(s)
     * @entrada P_CDCOMPRADOR: Cdigo do(s) comprador(es)
     * @entrada P_CDREGIAO:    Cdigo da(s) regio(es)
     * @entrada P_CDFORNECED:  Cdigo do(s) fornecedor(es)
     * @entrada P_DTINI:       Data inicial
     * @entrada P_DTFIN:       Data final
     * @entrada P_NRORG:       Cdigo da oranizao
     * @entrada P_NRPEDCOL:    Cdigo do pedido de coleta
     */
    PROCEDURE SAVEPEDCOL_PREVCONS(P_CDFILIAL    IN VARCHAR2,
                                  P_CDPRODINI   IN VARCHAR2,
                                  P_CDPRODFIN   IN VARCHAR2,
                                  P_CDMARCA     IN VARCHAR2,
                                  P_CDCOMPRADOR IN VARCHAR2,
                                  P_CDREGIAO    IN VARCHAR2,
                                  P_CDFORNECED  IN VARCHAR2,
                                  P_DTINI       IN DATE,
                                  P_DTFIN       IN DATE,
                                  P_NRORG       IN FILIAL.NRORG%TYPE,
                                  P_NRPEDCOL    IN PEDIDOCOLETA_TMP.NRPEDCOL%TYPE DEFAULT NULL);

    /*
     * Procedure responsvel inserir os registros do pedido de coleta em todas as tabelas do processo
     *
     * @entrada P_NRPEDCOL:       Cdigo do pedido de coleta
     * @entrada P_CDFILIAL:       Cdigo da unidade
     * @entrada P_CDFILIPROC:     Cdigo da unidade de processamento
     * @entrada P_QTMESES:        Quantidade de meses a considerar para estipular quantidades a comprar
     * @entrada P_DTEXPIRAPEDCOL: Data de expirao do pedido de coleta
     * @entrada P_IDPEDGERALREG:  Consolida todas as regies em um nico pedido de coleta por comprador
     * @entrada P_IDPEDGERALFORN: Consolida todos os fornecedores em um nico pedido de coleta por comprador
     * @entrada P_IDCURVAABCAF:   Considerar AF's do ltimo ms para estipular quantidades a comprar
     * @entrada P_IDCONSFORNAF:   Considera somente AF's do mesmo fornecedor
     * @saida   P_ERROR:          Cdigo/descrio possvel inconsistncia
     */
    PROCEDURE SAVEPEDIDOCOLETA(P_NRPEDCOL       IN PEDIDOCOLETA_TMP.NRPEDCOL%TYPE, 
                               P_CDFILIAL       IN FILIAL.CDFILIAL%TYPE,
                               P_CDFILIPROC     IN FILIAL.CDFILIAL%TYPE,
                               P_QTMESES        IN FLOAT,
                               P_DTEXPIRAPEDCOL IN DATE,
                               P_IDPEDGERALREG  IN VARCHAR2,
                               P_IDPEDGERALFORN IN VARCHAR2,
                               P_IDCURVAABCAF   IN VARCHAR2,
                               P_IDCONSFORNAF   IN VARCHAR2,
                               P_ERROR         OUT VARCHAR2);

    FUNCTION GETPEDCOLETA(P_IDPEDCOL         IN VARCHAR2,
                          P_NRCOTACAO        IN VARCHAR2,
                          P_COMPRADOR        IN VARCHAR2,
                          P_MARCA            IN VARCHAR2,
                          P_FORNECED         IN VARCHAR2,
                          P_REGIAO           IN VARCHAR2,
                          P_PRODINI          IN VARCHAR2,
                          P_PRODFIN          IN VARCHAR2,
                          P_CDFILIAL         IN VARCHAR2,
                          P_CDFILIPROC       IN VARCHAR2,
                          P_NRCOMPRA         IN VARCHAR2,
                          P_CDLEILAO         IN VARCHAR2,
                          P_NRSEQPRELIST     IN VARCHAR2,
                          P_CDOPERLANC       IN VARCHAR2,
                          P_DESCFORNECEDOR   IN VARCHAR2,
                          P_LISTFORNECEDOR   IN VARCHAR2,
                          P_QTMESES          IN VARCHAR2,
                          P_DTINI            IN DATE,
                          P_DTFIN            IN DATE,
                          P_DTEXPIRAPEDCOL   IN DATE,
                          P_BOPEDEMAIL       IN VARCHAR2,
                          P_IDPEDGERALREG    IN VARCHAR2,
                          P_IDPEDGERALFORN   IN VARCHAR2,
                          P_IDCURVAABCAF     IN VARCHAR2,
                          P_IDENVMERCADUM    IN VARCHAR2,
                          P_BOACRESITEM      IN VARCHAR2,
                          P_IDCONSFORNAF     IN VARCHAR2,
                          P_NRORG            IN VARCHAR2 DEFAULT '1') RETURN GETPEDCOLETA_TABLE PIPELINED;
END PEDIDO_COLETA;
/